<?php


class My_Order_Status { //Class Begins

    function __construct(){
        add_action( 'woocommerce_order_status_changed', array($this,'action_woocommerce_order_status_changed'), 20, 4);
        add_action( 'woocommerce_order_actions_end', array($this,'add_alert_on_action'));
        add_action( 'admin_enqueue_scripts', array($this,'my_custom_scripts'));
        add_action( 'wp_ajax_order_tax_refund', array($this, 'order_tax_refund'));
    }

    function add_alert_on_action(){
        /*
        As we are calling my_custom_scripts with callback argument '1', 
        this function is added because call_user_func_array() expects parameter 1 to be a valid callback,
        */
    }

    function order_tax_refund() {
        $order = new WC_Order($_POST['my_order_id']);
        $item_tax_mapping = [];
        foreach ($order->get_items() as $item_id => $item) {
            $item_tax_mapping[$item_id] = $item->get_meta('_item_tax_value');
        }
        wp_send_json($item_tax_mapping);
    }


    function my_custom_scripts() {
        wp_register_script( 'my_order_js', plugins_url( 'my_order_status.js', __FILE__ ), array( 'jquery' ) , '1', true);
        wp_enqueue_script( 'my_order_js' );
        wp_localize_script( 'my_order_js', 'ajaxurl', admin_url( 'admin-ajax.php' ) );
    }

    function action_woocommerce_order_status_changed($order_id, $status_transition_from, $status_transition_to, $instance){
        // only if previous status was 'on-hold'
        if ($this->is_request_to_rest_api() != 1 && $status_transition_from == 'on-hold'){
            $curl_client = curl_init();
            $quivers_sandbox = get_option('wc_settings_tab_demo_sandbox_status');
            $quivers_apikey = get_option('wc_settings_tab_demo_apikey');
            $options = $this->validateOrderAction($order_id, $quivers_sandbox, $quivers_apikey, $curl_client);
            $is_transition_valid = $options[0];
            $exist_on_quivers = $options[1];
            if (!$is_transition_valid){
                // revert back the status transition
                $order = new WC_Order($order_id);
                $order->update_status('on-hold');
                throw new Exception('Order status cannot be changed from '.$status_transition_from.' to '.$status_transition_to.' , because this order is already shipped by a Retailer.');
                echo "reacehd here";
            }
            if (!$exist_on_quivers){
            }
        }
        else{
        }
    }

    function is_request_to_rest_api() {
        if ( empty( $_SERVER['REQUEST_URI'] ) ) {
          return false;
        }
        $rest_prefix = trailingslashit( rest_get_url_prefix() );
        $woocommerce = ( false !== strpos( $_SERVER['REQUEST_URI'], $rest_prefix . 'wc/' ) );
        // Allow third party plugins use our authentication methods.
        $third_party = ( false !== strpos( $_SERVER['REQUEST_URI'], $rest_prefix . 'wc-' ) );
        return apply_filters( 'woocommerce_rest_is_request_to_rest_api', $woocommerce || $third_party );
      }

    function validateOrderAction($orderId, $quiversSandbox, $quiversAPIKey, $ch) {
        $startTime = microtime(true);
        $log = new My_WC_LogTracking();
        $orderUpdateValidateUrl = 'https://middleware.quiversdemo.com/profile/order/status';
        if($quiversSandbox == 'no') {
           $orderUpdateValidateUrl = "https://middleware.quivers.com/profile/order/status";
        }
        $payload = array(
           'client_type'=> 'Woocommerce',
           'profile_id'=> get_option('wc_middleware_profile_id'),
           'order_id'=> $orderId,
        );
        $log->session_start(["Order Data"=>$payload], date('r', $startTime)."UTC");
        curl_setopt($ch, CURLOPT_URL, $orderUpdateValidateUrl);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($payload));  //Post Fields
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $request = ["orderUpdateValidateUrl" => "https://middleware.quiversdemo.com/profile/order/status", "order_data" => json_encode($payload)];
        if($quiversSandbox == 'no') {
           $request = ["orderUpdateValidateUrl" => "https://middleware.quivers.com/profile/order/status", "order_data" => json_encode($payload)];
        }
        $headers = [
            'Content-Type: application/json',
            'Authorization: apikey '.$quiversAPIKey,
	        'Expect :'
        ];
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $server_output = curl_exec ($ch);
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close ($ch);
        $log->validate_order_action("HTTP Code*", $httpcode, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
        if($httpcode!=200) {
           return array(false, true);
        }
        $responseDecoded = json_decode($server_output, true);
        $endTime = microtime(true);
        $log->validate_order_action($request, $responseDecoded, date('r', $startTime)."UTC", date('r', $endTime)."UTC");
        if(empty($responseDecoded))
            return array(true, false);
        foreach ($responseDecoded as $key => $item) {
            if( $item['Value'] == 60 && $item['performed_by'] == 'Retailer' ) {
                return array(false, true);
            }
        }
       return array(true, true);
   }
}

if( class_exists("My_Order_Status") ) $GLOBALS['my_order_status'] = new My_Order_Status();

?>
